@@ -18,7 +18,7 @@ from typing import Optional, Union, Iterable, Callable, List, Generator
 
 from loguru import logger
 
-from bricks import state
+from bricks import state, const
 from bricks.core import dispatch, signals, events
 from bricks.core.context import Flow, Error
 from bricks.core.events import EventManager
@@ -41,6 +41,7 @@ class Context(Flow):
             self,
             target: "Spider",
             form: str = state.const.ON_CONSUME,
+            division: Optional[bool] = False,
             **kwargs
 
     ) -> None:
@@ -52,6 +53,7 @@ class Context(Flow):
         self.queue_name: str = kwargs.pop("queue_name", f'{self.__class__.__module__}.{self.__class__.__name__}')
         super().__init__(form, target, **kwargs)
         self.target: Spider = target
+        self.division: bool = division
 
     def __setattr__(self, key, value):
         if key == "seeds" and type(value) is not Item:
@@ -88,9 +90,15 @@ class Context(Flow):
         :return:
         """
         ret = self.task_queue.replace(self.queue_name, (self.seeds, new), qtypes=qtypes)
-        self.seeds.fingerprint = new
+        self.seeds = new
         return ret
 
+    def divisive(self, qtypes=("temp",)):
+        new = {**self.seeds, "$division": str(uuid.uuid4())}
+        self.task_queue.put(self.queue_name, new, qtypes=qtypes)
+        self.seeds = new
+        return new
+
     def submit(self, *obj: Union[Item, dict], call_later=False, attrs: dict = None) -> List["Context"]:
         """
         专门为新请求分支封装的方法, 会自动将请求放入队列
@@ -298,10 +306,9 @@ class Spider(Pangu):
             gen = [gen]
 
         for seeds in gen:
-            stuff: InitContext = context.copy()
-            stuff.seeds = seeds
-            stuff.flow({"next": self.produce_seeds})
-            self.on_consume(stuff)
+            ctx: InitContext = context.copy()
+            ctx.flow({"next": self.produce_seeds, "seeds": seeds})
+            self.on_consume(ctx)
 
         record.update(finish=str(datetime.datetime.now()))
         task_queue.command(queue_name, {"action": task_queue.COMMANDS.RELEASE_INIT, "time": int(time.time() * 1000)})
@@ -325,15 +332,14 @@ class Spider(Pangu):
             settings['success_size'] - settings['success'],
             len(seeds)
         ])]
-        context.seeds = seeds
+        context.update({
+            "maxsize": settings['queue_size'],
+            "seeds": seeds,
+        })
         fettle = pandora.invoke(
             func=self.put_seeds,
             args=[context.seeds],
             kwargs={
-                "maxsize": settings['queue_size'],
-                "task_queue": context.task_queue,
-                "queue_name": context.queue_name,
-                "priority": context.priority,
                 "where": "init"
             },
             annotations={InitContext: context},
@@ -627,15 +633,30 @@ class Spider(Pangu):
             if ret is None: raise signals.Empty
             context.seeds = ret
             context.form = state.const.AFTER_GET_SEEDS
-            events.EventManager.invoke(context)
+            events.EventManager.invoke(
+                context,
+                annotations={
+                    Context: context
+                },
+                namespace={
+                    "context": context,
+                    "seeds": context.seeds
+                }
+            )
 
             return ret
 
         return wrapper
 
     def on_seeds(self, context: Context):
-        request = self.make_request(context)
-        context.flow({"request": request})
+        for index, stuff in enumerate(pandora.iterable(self.make_request(context))):
+            if index == 0:
+                ctx = context
+            else:
+                ctx = context.branch()
+                ctx.division and ctx.divisive()
+
+            ctx.flow({"request": stuff})
 
     def on_retry(self, context: Context):
         """
@@ -716,33 +737,78 @@ class Spider(Pangu):
         else:
             response: Response = self.downloader.fetch(context.request)
 
-        context.flow({"response": response})
         return response
 
     def _when_on_request(self, raw_method):  # noqa
+
         @functools.wraps(raw_method)
         def wrapper(context: Context, *args, **kwargs):
+
             context.form = state.const.BEFORE_REQUEST
-            events.EventManager.invoke(context)
-            context.form = state.const.ON_REQUEST
-            self.number_of_total_requests.increment()
-            pandora.invoke(
-                func=raw_method,
-                args=args,
-                kwargs=kwargs,
-                annotations={
-                    self.Context: context,
-                    Item: context.seeds,
-                    Request: context.request,
-                },
-                namespace={
-                    "context": context,
-                    "seeds": context.seeds,
-                    "request": context.request
-                }
-            )
-            context.form = state.const.AFTER_REQUEST
-            events.EventManager.invoke(context)
+            try:
+                events.EventManager.invoke(
+                    context,
+                    annotations={
+                        self.Context: context,
+                        Item: context.seeds,
+                        Request: context.request,
+                    },
+                    namespace={
+                        "context": context,
+                        "seeds": context.seeds,
+                        "request": context.request
+                    }
+                )
+            except signals.Switch as jump:
+                if jump.by == 'func':
+                    raise
+                else:
+                    gen = context.response
+
+            else:
+                context.form = state.const.ON_REQUEST
+                self.number_of_total_requests.increment()
+                gen = pandora.invoke(
+                    func=raw_method,
+                    args=args,
+                    kwargs=kwargs,
+                    annotations={
+                        self.Context: context,
+                        Item: context.seeds,
+                        Request: context.request,
+                    },
+                    namespace={
+                        "context": context,
+                        "seeds": context.seeds,
+                        "request": context.request
+                    }
+                )
+            for index, stuff in enumerate(pandora.iterable(gen)):
+                stuff.request = context.request if stuff.request is ... else stuff.request
+                if index == 0:
+                    ctx = context
+                else:
+                    ctx = context.branch()
+                    ctx.division and ctx.divisive()
+
+                ctx.flow({"response": stuff, "request": stuff.request}, ctx.next == self.on_request)
+                events.EventManager.next(
+                    ctx,
+                    form=state.const.AFTER_REQUEST,
+                    annotations={
+                        self.Context: ctx,
+                        Item: ctx.seeds,
+                        Request: ctx.request,
+                        Response: ctx.response
+                    },
+                    namespace={
+                        "context": ctx,
+                        "seeds": ctx.seeds,
+                        "request": ctx.request,
+                        "response": ctx.response,
+                    },
+                    callback=lambda mctx: mctx.rollback(recursion=False)
+                )
 
         return wrapper
 
@@ -784,14 +850,13 @@ class Spider(Pangu):
         else:
             items = prepared.func(*prepared.args, **prepared.kwargs)
 
-        context.flow({"items": items})
         return items
 
     def _when_on_response(self, raw_method):  # noqa
         @functools.wraps(raw_method)
         def wrapper(context: Context, *args, **kwargs):
             context.form = state.const.ON_PARSE
-            pandora.invoke(
+            gen = pandora.invoke(
                 func=raw_method,
                 args=args,
                 kwargs=kwargs,
@@ -808,6 +873,15 @@ class Spider(Pangu):
                     "seeds": context.seeds,
                 }
             )
+            if isinstance(gen, (list, Items)): gen = [gen]
+            for index, stuff in enumerate(pandora.iterable(gen)):
+                if index == 0:
+                    ctx = context
+                else:
+                    ctx = context.branch()
+                    ctx.division and ctx.divisive()
+
+                ctx.flow({"items": stuff}, flag=ctx.next == self.on_response)
 
         return wrapper
 
@@ -851,13 +925,27 @@ class Spider(Pangu):
         else:
             prepared.func(*prepared.args, **prepared.kwargs)
 
-        context.flow()
-
     def _when_on_pipeline(self, raw_method):  # noqa
         @functools.wraps(raw_method)
         def wrapper(context: Context, *args, **kwargs):
             context.form = state.const.BEFORE_PIPELINE
-            events.EventManager.invoke(context)
+            events.EventManager.invoke(
+                context,
+                annotations={
+                    Response: context.response,
+                    Request: context.request,
+                    Items: context.items,
+                    Item: context.seeds,
+                    Context: context
+                },
+                namespace={
+                    "context": context,
+                    "request": context.request,
+                    "response": context.response,
+                    "seeds": context.seeds,
+                    "items": context.items,
+                }
+            )
             context.form = state.const.ON_PIPELINE
             prepared = pandora.prepare(
                 func=raw_method,
@@ -880,7 +968,24 @@ class Spider(Pangu):
             )
             prepared.func(*prepared.args, **prepared.kwargs)
             context.form = state.const.AFTER_PIPELINE
-            events.EventManager.invoke(context)
+            events.EventManager.invoke(
+                context,
+                annotations={
+                    Response: context.response,
+                    Request: context.request,
+                    Items: context.items,
+                    Item: context.seeds,
+                    Context: context
+                },
+                namespace={
+                    "context": context,
+                    "request": context.request,
+                    "response": context.response,
+                    "seeds": context.seeds,
+                    "items": context.items,
+                }
+            )
+            context.flow(flag=context.next == self.on_pipeline)
 
         return wrapper
 
@@ -916,7 +1021,7 @@ class Spider(Pangu):
             state.const.AFTER_REQUEST,
             {"func": on_request.After.show_response},
             {"func": on_request.After.conditional_scripts},
-            {"func": on_request.After.is_success}
+            {"func": on_request.After.bypass}
         )
 
     @pandora.Method
@@ -1012,23 +1117,21 @@ class Spider(Pangu):
             listener.recv(future_id, self)
             return super(self.__class__, self).success(shutdown)
 
-        def mock_on_retry(self):
-            future_id = self.seeds.get('$futureID')
-            future_max_retry = self.seeds.get('$futureMaxRetry')
-            counter = listener.counter[future_id]
-            times = next(counter)
-            if times + 1 >= future_max_retry:
-                listener.counter.pop(future_id, None)
-                return self.success(shutdown=True)
-            else:
-                return super(self.__class__, self).retry()
-
         def mock_on_request(self, context: Context):
             future_type = context.seeds.get('$futureType', "$response")
             if future_type == '$request':
                 raise signals.Success
             return super(self.__class__, self).on_request(context)
 
+        def set_max_retry(context: Context):
+            future_id = context.seeds.get('$futureID')
+            future_max_retry = context.seeds.get('$futureMaxRetry')
+            counter = listener.counter[future_id]
+            times = next(counter)
+            if times >= future_max_retry:
+                listener.counter.pop(future_id, None)
+                raise signals.Success
+
         def mock_on_response(self, context: Context):
             future_type = context.seeds.get('$futureType', "$response")
 
@@ -1053,8 +1156,9 @@ class Spider(Pangu):
         })
         clazz = type("Listen", (cls,), modded)
 
-        clazz.Context = type("ListenContext", (cls.Context,), {"success": mock_on_success, "retry": mock_on_retry})
+        clazz.Context = type("ListenContext", (cls.Context,), {"success": mock_on_success})
         listen: Spider = clazz(**attrs)
+        listen.use(const.BEFORE_REQUEST, {"func": set_max_retry, "index": -math.inf})
         listener = Listener(listen)
         return listener.run()
 
